/*     */ package org.apache.batik.extension.svg;
/*     */ 
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.util.Map;
/*     */ import org.apache.batik.bridge.AbstractSVGFilterPrimitiveElementBridge;
/*     */ import org.apache.batik.bridge.Bridge;
/*     */ import org.apache.batik.bridge.BridgeContext;
/*     */ import org.apache.batik.bridge.BridgeException;
/*     */ import org.apache.batik.bridge.SVGUtilities;
/*     */ import org.apache.batik.ext.awt.image.PadMode;
/*     */ import org.apache.batik.ext.awt.image.renderable.Filter;
/*     */ import org.apache.batik.ext.awt.image.renderable.PadRable8Bit;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.w3c.dom.Element;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BatikHistogramNormalizationElementBridge
/*     */   extends AbstractSVGFilterPrimitiveElementBridge
/*     */   implements BatikExtConstants
/*     */ {
/*  54 */   public String getNamespaceURI() { return "http://xml.apache.org/batik/ext"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  61 */   public String getLocalName() { return "histogramNormalization"; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  68 */   public Bridge getInstance() { return new BatikHistogramNormalizationElementBridge(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Filter createFilter(BridgeContext ctx, Element filterElement, Element filteredElement, GraphicsNode filteredNode, Filter inputFilter, Rectangle2D filterRegion, Map filterMap) {
/*     */     Rectangle2D rectangle2D;
/*  98 */     Filter in = AbstractSVGFilterPrimitiveElementBridge.getIn(filterElement, filteredElement, filteredNode, inputFilter, filterMap, ctx);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 104 */     if (in == null) {
/* 105 */       return null;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 111 */     Filter sourceGraphics = (Filter)filterMap.get("SourceGraphic");
/*     */     
/* 113 */     if (in == sourceGraphics) {
/* 114 */       rectangle2D = filterRegion;
/*     */     } else {
/* 116 */       rectangle2D = in.getBounds2D();
/*     */     } 
/*     */     
/* 119 */     Rectangle2D primitiveRegion = SVGUtilities.convertFilterPrimitiveRegion(filterElement, filteredElement, filteredNode, rectangle2D, filterRegion, ctx);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 127 */     float trim = 1.0F;
/* 128 */     String s = filterElement.getAttributeNS(null, "trim");
/*     */ 
/*     */     
/* 131 */     if (s.length() != 0) {
/*     */       try {
/* 133 */         trim = SVGUtilities.convertSVGNumber(s);
/*     */       } catch (NumberFormatException nfEx) {
/* 135 */         throw new BridgeException(ctx, filterElement, nfEx, "attribute.malformed", new Object[] { "trim", s });
/*     */       } 
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 141 */     if (trim < 0.0F) { trim = 0.0F; }
/* 142 */     else if (trim > 100.0F) { trim = 100.0F; }
/*     */     
/* 144 */     Filter filter = in;
/* 145 */     BatikHistogramNormalizationFilter8Bit batikHistogramNormalizationFilter8Bit = new BatikHistogramNormalizationFilter8Bit(filter, trim / 100.0F);
/*     */     
/* 147 */     PadRable8Bit padRable8Bit = new PadRable8Bit(batikHistogramNormalizationFilter8Bit, primitiveRegion, PadMode.ZERO_PAD);
/*     */ 
/*     */     
/* 150 */     AbstractSVGFilterPrimitiveElementBridge.updateFilterMap(filterElement, padRable8Bit, filterMap);
/*     */ 
/*     */     
/* 153 */     AbstractSVGFilterPrimitiveElementBridge.handleColorInterpolationFilters(padRable8Bit, filterElement);
/*     */     
/* 155 */     return padRable8Bit;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected static int convertSides(Element filterElement, String attrName, int defaultValue, BridgeContext ctx) {
/* 173 */     String s = filterElement.getAttributeNS(null, attrName);
/* 174 */     if (s.length() == 0) {
/* 175 */       return defaultValue;
/*     */     }
/* 177 */     int ret = 0;
/*     */     try {
/* 179 */       ret = SVGUtilities.convertSVGInteger(s);
/*     */     } catch (NumberFormatException nfEx) {
/* 181 */       throw new BridgeException(ctx, filterElement, nfEx, "attribute.malformed", new Object[] { attrName, s });
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 186 */     if (ret < 3) {
/* 187 */       throw new BridgeException(ctx, filterElement, "attribute.malformed", new Object[] { attrName, s });
/*     */     }
/*     */     
/* 190 */     return ret;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-extension.jar!\org\apache\batik\extension\svg\BatikHistogramNormalizationElementBridge.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */